/*
 * Class.java
 *
 * Created on February 28, 2002, 2:40 PM
 */

import java.io.*;
import javax.swing.*;
import javax.swing.table.*;

/**
 * Kapselt die komplette Benutzerverwaltung, wird von der GUI aufgerufen und implementiert daf?r das <code>TableModel</code>-Interface.
 * @stereotype control 
 * @author Christian Blichmann
 * @see SoPraContentManagement
 */
public class UserManagement extends AbstractTableModel implements Streamable,
    TableModel {
    /** Attribute */
    /**
     * Interne Referenz auf eine Instanz von <code>UserTable</code>.
     * @label verwaltet
     * @clientCardinality 1
     * @supplierCardinality 1 
     * @link aggregation
     */
    private UserTable myUserTable;
    
    /** Initialisiert die Benutzerverwaltung */
    public UserManagement() {}
    
    /**
     * Erzeugt einen neuen Benutzer.
     * @param name - Nachname
     * @param firstName - Vorname
     * @param eMail - Emailadresse
     * @param login - Loginname
     * @param password - Passwort
     * @param adminUser - Verwalter oder nicht
     * @return <code>true</code> wenn erfolgreich und Benutzer noch nicht
     *         vorhanden, <code>false</code> sonst 
     * pre: Alle Parameter muessen ausgefuellt sein, myUserTable != null
     * post: Es wurde ein Benutzer in myUserTable angelegt
     */
    public boolean createUser(String name, String firstName, String eMail,
        String login, String password, boolean adminUser) {
        return myUserTable.createUser(name, firstName, eMail, login, password,
            adminUser);
    }
    
    /**
     * Prueft, ob der Benutzer Verwalterrechte besitzt.
     * @param login - Loginname
     * @return <code>true</code>, falls Verwalter, <code>false</code>,
     *         falls nicht
     * pre: login muss ausgefuellt sein, Benutzer mit Loginname login muss existieren
     * post: Benutzer ist Verwalter, oder nicht
     */
    public boolean isAdminUser(String login) {
        return myUserTable.findUser(login).isAdminUser();
    }

    public String getFullNameUser(String login) {
        User user = myUserTable.findUser(login);
        String result = "";

        if (user != null)
            result = user.getFirstName() + " " + user.getName();
        
        return result;
    }

    public String getEMailUser(String login) {
        User user = myUserTable.findUser(login);
        String result = "";

        if (user != null)
            result = user.getEMail();
        
        return result;
    }

    /**
     * Prueft, ob der Benutzer existiert.
     * @param login - Loginname
     * @return <code>true</code>, falls Benutzer existiert, <code>false</code>,
     *         falls nicht
     * pre: login muss ausgefuellt sein
     * post: Benutzer existiert, oder nicht
     */
    public boolean existUser(String login) {
        return myUserTable.findUser(login) != null;
    }

    /**
     * L?scht einen Benutzer.
     * @return <code>true</code> wenn Benutzer gel?scht, <code>false</code> wenn
     *         Benutzer nicht vorhanden oder letzter/aktueller Benutzer. 
     * pre: Benutzer mit Loginname login muss existieren, myUserTable != null
     * post: Der Benutzer wurde aus myUserTable entfernt
     */
    public boolean deleteUser(String login) { return myUserTable.deleteUser(login); }

    /**
     * Loggt den Benutzer, der sich mit <code>login</code> und <code>password
     * </code> identifiziert hat, ein und setzt <code>myCurUserLogin</code> auf
     * den Wert von <code>login</code>.
     * @param login - Loginname
     * @param password - Passwort
     * @return <code>true</code> falls erfolgreich, <code>false</code> sonst. 
     * pre: login und password muessen ausgefuellt sein, myUserTable != null
     * post: Der Benutzer mit Loginname und Passwort wurde eingeloggt oder nicht
     */
    public boolean loginUser(String login, String password) {
        User myUser = myUserTable.findUser(login); // Benutzer in Tabelle suchen

        // Falls Benutzer gefunden (nicht null), Passwort vergleichen.
        boolean result = myUser != null && myUser.getPassword().equals(password);
        
        return result;
    }
    
    /** Streamable Interface */
    /** pre und post: siehe Streamable */
    public void streamIn(IOControl ioCtl) {
        myUserTable = (UserTable) ioCtl.readUsers();
        
        if (myUserTable == null)
            myUserTable = new UserTable();
        else if (myUserTable.size() == 0)
            myUserTable.createUser("Gonzales", "Speedy", "speedy@gonzales.net",
                "tequilla", "sunrise", true);
    }
    
    /** pre und post: siehe Streamable */
    public void streamOut(IOControl ioCtl) {
        ioCtl.writeUsers(myUserTable);
    }

    /** TableModel Interface (siehe JDK 1.4.0) */
    static final String[] columnNames = {
        "Name", "Vorname", "Email", "Login", "Kategorie"
    };

    public String getColumnName(int col) { 
        return columnNames[col]; 
    }
    
    public int getRowCount() { return myUserTable.size(); }
    public int getColumnCount() { return columnNames.length; }
    
    public Object getValueAt(int row, int col) {
        User myUser = (User) myUserTable.elementAt(row);
        
        switch (col) {
            case 0: return myUser.getName();
            case 1: return myUser.getFirstName();
            case 2: return myUser.getEMail();
            case 3: return myUser.getLogin();
            case 4: return myUser.isAdminUser() ? "Verwalter" : "Gruppenbetreuer";
            case 5: return myUser.getPassword();
            default: return null;
        }
    }
    
    public boolean isCellEditable(int row, int col) { return false; }
    
    public void setValueAt(Object value, int row, int col) {
        User myUser = (User) myUserTable.elementAt(row);
        
        switch (col) {
            case 0:
                myUser.setName((String) value);
                break;
            case 1:
                myUser.setFirstName((String) value);
                break;
            case 2:
                myUser.setEMail((String) value);
                break;
            case 3:
                myUser.setLogin((String) value);
                break;
            case 4:
                myUser.setAdminUser(((java.lang.Boolean) value).booleanValue());
                break;
            case 5:
                myUser.setPassword((String) value);
                break;
        }
    }
}